import java.util.*;


/*
(Table 1)
         Creature Spells  Total
Green	    34     15      90
White	    34     15      90
Artifacts   20	    15	   35
Gold	    10	    10	   20
Land			   20
Total	    70      80	   150

(Table 2)
      common	uncommon  rare  total
Green	 34	   15      32	 90
White	 34	   15      32	 90
Artifacts 5          2     7     12
Gold      15         10    10    45
Land     10         5      5     20
Total    40        50      70   150
*/


public class GenerateStatistics
{
  final private String newLine = "\r\n";
  final private String tab = "\t";


  private IO_Read ioRead;
  private HashMap mapToCardColor = new HashMap();

  private String[] rowColor =
  {
    "White", "Blue", "Black", "Red", "Green", "Artifact", "Gold", "Land"
  };


  public GenerateStatistics(IO_Read io)
  {
    mapToCardColor.put("White", CardColor.White);
    mapToCardColor.put("Blue", CardColor.Blue);
    mapToCardColor.put("Black", CardColor.Black);
    mapToCardColor.put("Red", CardColor.Red);
    mapToCardColor.put("Green", CardColor.Green);
    mapToCardColor.put("Artifact", CardColor.Colorless);
    mapToCardColor.put("Gold", CardColor.Multi);
    mapToCardColor.put("Land", CardColor.Land);
  }

  public String getStatistics()
  {
    ioRead = new IO_Read();

    return getTable1() +newLine +newLine +getTable2();
  }

  //table 2 shows rarity by color
  public String getTable2()
  {
    ArrayList nameList = getOnlyLimitedCardNames();

    String s = tab +"Common" +tab +"Uncommon" +tab +"Rare" +tab +"Total" +newLine;

    int array[];
    String colorType;

    for(int i = 0; i < rowColor.length; i++)
    {
      colorType = mapToCardColor.get(rowColor[i]).toString();
      array = getColor_table2(colorType, nameList);

      s += rowColor[i] +tab +array[0] +tab +array[1] +tab +array[2] +tab +(array[0] + array[1] + array[2]) +newLine;
    }

    //total all rows, do creature and spell totals
    array = getColor_table2("all", nameList);
    s += "Total" +tab +array[0] +tab +array[1] +tab +array[2] +tab +(array[0] + array[1] + array[2]) +newLine;

    return s;
  }//getTable2()

  //colorType is from CardColor, like CardColor.White
  //colorType can be "all" which will count all of Arraylist allCardNames
  //table 2 shows rarity by color
  //returns int[0] is the number of commons
  //returns int[1] is the number of uncommons
  //returns int[2] is the number of rares
  private int[] getColor_table2(String colorType, ArrayList allCardNames)
  {
    int nCommon = 0;
    int nUncommon = 0;
    int nRare = 0;

    String out = "";
    String name;
    ArrayList info;

    for(int i = 0; i < allCardNames.size(); i++)
    {
      name = allCardNames.get(i).toString();
      info = ioRead.getCardInfo(name);

      //is the right color?
      if(colorType.equals(CardColor.getType(info)) || colorType.equals("all"))
      {
        if(ioRead.isCommon(name))
          nCommon++;

        if(ioRead.isUncommon(name))
          nUncommon++;

        if(ioRead.isRare(name))
          nRare++;
      }
    }//for - all card names

    return new int[] {nCommon, nUncommon, nRare};
  }//getColor_table2()


  //table 1 shows creatures and spells by color
  public String getTable1()
  {
    ArrayList nameList = getOnlyLimitedCardNames();

    String s = tab +"Creatures" +tab +"Spells" +tab +"Total" +newLine;

    int array[];
    String colorType;
    //-1, do not include land, we only care about land totals
    for(int i = 0; i < rowColor.length - 1; i++)
    {
      colorType = mapToCardColor.get(rowColor[i]).toString();
      array = getColor_table1(colorType, nameList);

      s += rowColor[i] +tab +array[0] +tab +array[1] +tab +(array[0] + array[1]) +newLine;
    }

    //do land totals
    colorType = mapToCardColor.get("Land").toString();
    array = getColor_table1(colorType, nameList);
    s += "Land" +tab +tab +tab +(array[0] + array[1]) +newLine;

    //total all rows, do creature and spell totals
    array = getColor_table1("all", nameList);
    s += "Total" +tab +array[0] +tab +array[1] +tab +(array[0] + array[1]) +newLine;


    return s;
  }//getStatistics()

  //colorType is from CardColor, like CardColor.White
  //colorType can be "all" which will count all of Arraylist allCardNames
  //table 1 is only creature and spells
  //returns int[0] is the number of creatures
  //returns [1] is the number of spells (non-creatures)
  private int[] getColor_table1(String colorType, ArrayList allCardNames)
  {
    int nCreature = 0;
    int nSpells = 0;

    String out = "";
    String name;
    ArrayList info;

    for(int i = 0; i < allCardNames.size(); i++)
    {
      name = allCardNames.get(i).toString();
      info = ioRead.getCardInfo(name);

      //is the right color?
      if(colorType.equals(CardColor.getType(info)) || colorType.equals("all"))
      {
        if(isCreature(info))
          nCreature++;
        else
          nSpells++;
      }
    }//for - all card names

    return new int[] {nCreature, nSpells};
  }//getColor_table1()

  //returns only cards that are the specified color
  //colorType is from CardColor, like CardColor.Green
  private ArrayList getColor(String colorType, ArrayList allCardNames)
  {
    ArrayList out = new ArrayList();
    String name;
    ArrayList info;

    for(int i = 0; i < allCardNames.size(); i++)
    {
      name = allCardNames.get(i).toString();
      info = ioRead.getCardInfo(name);

      if(colorType.equals(CardColor.getType(info)))
         out.add(name);
    }
    return out;
  }//getColor()

  //returns true if Card is a creature
  //ArrayList listCardInfo is from IO_Read.getCardInfo()
  private boolean isCreature(ArrayList listCardInfo)
  {
    String type = listCardInfo.get(2).toString();
    return 0 <= type.indexOf("Creature");
  }

  //ioRead.getAllCardNames() returns all card names in cards.txt
  //we only care about card names that are in
  //common.txt, uncommon.txt, rare.txt
  private ArrayList getOnlyLimitedCardNames()
  {
    ArrayList out = new ArrayList();
    ArrayList all = ioRead.getAllCardNames();

    String n;
    for(int i = 0; i < all.size(); i++)
    {
      n = all.get(i).toString();

      //see if the card name is common, uncommon, or rare
      if(ioRead.isCommon(n) || ioRead.isUncommon(n) || ioRead.isRare(n))
        out.add(n);
    }

    return out;
  }//getOnlyLimitedCardNames()
}
