import java.util.*;

public class GuiTable_Sorter implements Comparator
{
  public static final String CardName = "Card Name";
  public static final String Type = "Type";//artifact, artifact creature, creature, instant, land, planeswalkers, sorcery
  public static final String Rarity = "Rarity"; //C,U,R,none
  public static final String Color = "Color"; // WUBRG, multicolored, land, artifact


  private String sortColumn;
  private IO_Read cardDatabase;
  private GuiTable_Rarity rarity;
  private HashMap typeMap = new HashMap();

  public GuiTable_Sorter(IO_Read in_cardDatabase, GuiTable_Rarity in_rarity)
  {
    cardDatabase = in_cardDatabase;
    rarity = in_rarity;

    sortColumn = CardName;

    setupTypeMap();
  }

  public void sortBy(String sortBy)
  {
    sortColumn = sortBy;
  }

  public int compare(Object in_a, Object in_b)
  {
    Comparable a = null;
    Comparable b = null;

    if(CardName.equals(sortColumn))
    {
      a = (String) in_a;
      b = (String) in_b;
    }
    else if(Type.equals(sortColumn))
    {
      a = getType((String) in_a);
      b = getType((String) in_b);
    }
    else if(Rarity.equals(sortColumn))
    {
      a = getRarity((String) in_a);
      b = getRarity((String) in_b);
    }
    else if(Color.equals(sortColumn))
    {
      a = getColor((String) in_a);
      b = getColor((String) in_b);
    }

    if(a == null || b == null)
      Error.report("GuiTable_Sorter : compare() error, a or b is null");

    return a.compareTo(b);
  }//compare()

  //IO_read.getCardInfo(String cardName) description
  //ArrayList list = new ArrayList()
  //list.add("Shock")    0
  //list.add("R")        1
  //list.add("Instant")  2
  //list.add("Deals 2 damage to target creature or player")  3
  private String getColor(String cardName)
  {
    String m = (String) cardDatabase.getCardInfo(cardName).get(1);

    //remove duplicate colors, "G" and "G" is reduced to just "G"
    //colors.add("G")
    //colors.add("G")
    //colors.size() is just 1
    Set colors = new HashSet();

    for(int i = 0; i < m.length(); i++)
    {
      switch(m.charAt(i))
      {
      case ' ': break;
      case 'W': colors.add("A"); break;
      case 'U': colors.add("B"); break;
      case 'B': colors.add("C"); break;
      case 'R': colors.add("D"); break;
      case 'G': colors.add("E"); break;
      }
    }//for

    //is multi-colored?
    if(1 < colors.size())
      return "X";

    //is single colored?
    if(1 == colors.size())
      return colors.iterator().next().toString();

    //is land?
    String type = (String) cardDatabase.getCardInfo(cardName).get(2);
    if(0 <= type.indexOf("Land"))
      return "Y";


    //this is for artifacts and anything else that might be an error
    //like Crimson Kobolds
    return "Z";
  }//getColor()


  private Comparable getRarity(String cardName)
  {
    if(rarity.isCommon(cardName))
      return new Integer(0);

    if(rarity.isUncommon(cardName))
      return new Integer(1);

    if(rarity.isRare(cardName))
      return new Integer(2);

    return new Integer(3);
  }

  private Comparable getType(String cardName)
  {
    Iterator it = typeMap.keySet().iterator();
    ArrayList list = cardDatabase.getCardInfo(cardName);
    String type = list.toString();

    String check;

    while(it.hasNext())
    {
      check = it.next().toString();

      if(0 <= type.indexOf(check))
        return typeMap.get(check).toString();
    }

    //Error.report("GuiTable_Sorter : getType() error, type not found - " +type +" : card name= " +cardName);

    //can't presume HashSet is ordered, so just "Artifacts" are 3
    //while "Artifact Creatures" are 2
    //this is a wierd line of code but it works
    return "3"; //artifact
  }//getType()


  private void setupTypeMap()
  {
    //planeswalkers, artifact creature, artifact,
    //creature, instant, land, sorcery
    typeMap.put("Planeswalker", "1");
    typeMap.put("Artifact Creature", "2");
    //typeMap.put("Artifact", "3");
    typeMap.put("Creature", "4");
    typeMap.put("Instant", "5");
    typeMap.put("Land", "6");
    typeMap.put("Sorcery", "7");
    typeMap.put("Enchantment", "8");
  }//setupTypeMap()
}