import java.io.*;
import java.util.*;
import javax.swing.JPanel;

public class IO_Read
{
  //private final File path;

  //key - String cardName, data - ArrayList, see getCardInfo() for an example
  private HashMap cardTextMap   = new HashMap();

  private HashSet commonList;
  private HashSet uncommonList;
  private HashSet rareList;

  public IO_Read()
  {
    this(IO_Util.getCurrentDirectory());
  }

  public IO_Read(String directory)
  {
    /* Don't check the directory, the class Filename does that

    path = new File(directory);

    if(! path.isDirectory() || ! path.exists())
      Error.report("IO_Read : constructor() error, path is not a directory - " +path.getAbsolutePath());
    */

    readFiles();

    internal_testFiles();//must be called after readFile();
  }

  //make a copy in case the return value is changed
  //need to make a copy to keep all private variables private
  public HashSet getCommon() {return new HashSet(commonList);}
  public HashSet getUncommon() {return new HashSet(uncommonList);}
  public HashSet getRare() {return new HashSet(rareList);}

  public String getRarity(String cardName)
  {
    if(isCommon(cardName))
      return "common";

    if(isUncommon(cardName))
      return "uncommon";

    if(isRare(cardName))
      return "rare";

    return "";
  }

  //test to see that all cards are in cards.txt
  //this checks getAllCardNames(), commonList, uncommonList, rareList
  //by calling getCardInfo() for all of the cards
  private void internal_testFiles()
  {
    //just for testing, get text for all cards
    //if there is an error, it will be displayed
    ArrayList list = this.getAllCardNames();
    for(int i = 0; i < list.size(); i++)
      if(! cardTextMap.containsKey((String) list.get(i)))
        Error.report("IO_Read : internal_testFiles() error, card name not found in cards.txt: " +list.get(i));

    //get all card text for commonList, uncommonList, rareList
    list = new ArrayList(commonList);
    for(int i = 0; i < list.size(); i++)
      if(! cardTextMap.containsKey((String) list.get(i)))
        Error.report("IO_Read : internal_testFiles() error, card name not found in cards.txt, this card was in common.txt: " +list.get(i));

    list = new ArrayList(uncommonList);
    for(int i = 0; i < list.size(); i++)
      if(! cardTextMap.containsKey((String) list.get(i)))
        Error.report("IO_Read : internal_testFiles() error, card name not found in cards.txt, this card was in uncommon.txt: " +list.get(i));

    list = new ArrayList(rareList);
    for(int i = 0; i < list.size(); i++)
      if(! cardTextMap.containsKey((String) list.get(i)))
        Error.report("IO_Read : internal_testFiles() error, card name not found in cards.txt, this card was in rare.txt: " +list.get(i));

  }//internal_testFiles();

  private void readFiles()
  {
    //maybe HashSet is faster than ArrayList, see getRarity()
    commonList   = readCardNames("common.txt");
    uncommonList = readCardNames("uncommon.txt");
    rareList     = readCardNames("rare.txt");

    cardTextMap = readCardsTxt();
  }//readFiles()


  //reads cards.txt, initializes cardTextMap
  //cards.txt ends with "End", which is read as a card name
  //but it is never added
  //cards are only added to cardTextMap when a blank line is found
  //basically readCardsTxt() doesn't do anything with "End" but it works
  private HashMap readCardsTxt()
  {
    HashMap map = new HashMap();

    File file = new File(Filename.getPathFor("cards.txt"));
//    File file = new File(path, "cards.txt");

    try{
      BufferedReader io = new BufferedReader(new FileReader(file));
      ArrayList cardText = new ArrayList();

      String line = io.readLine();
      while(line != null)
      {
        line = line.trim();

        //is separator?
        //card text is separated by a blank line
        if(line.length() == 0)
        {
          //this might occur if there are multiple blank lines
          //at the end of cards.txt
          if(! cardText.isEmpty())
          {
            addToMap(map, cardText);
            cardText = new ArrayList();
          }
        }
        else
        {
          cardText.add(line);
        }

        line = io.readLine();
      }
    }//try
    catch(Exception ex)
    {
      Error.report("IO_Read : readCardsTxt() error, problem with cards.txt " +file.getAbsolutePath() +" : " +ex);
    }

    return map;
  }//readCardsTxt()

  //ArrayList list, for example see getCardInfo()
  private void addToMap(HashMap map, ArrayList list)
  {
    //remove boring stuff
    if(list.contains("no text"))
      list.remove("no text");

    String cardName = (String) list.get(0);
    map.put(cardName, list);
  }

  //rarityFile should be "common.txt" or "uncommon.txt" or "rare.txt"
  private HashSet readCardNames(String rarityFile)
  {
    File file = new File(Filename.getPathFor(rarityFile));
    ArrayList nameList = new ArrayList();

    try{
      BufferedReader io = new BufferedReader(new FileReader(file));

      String line = io.readLine();
      while(line != null)
      {
        line = line.trim();

        //skip comments and blank lines
        if(! line.startsWith("//") && line.length() != 0)
          nameList.add(line);

        line = io.readLine();
      }
    }//try
    catch(Exception ex)
    {
      Error.report("IO_Read : readCardNames() error, trying to read file " +file.getAbsolutePath() +" : " +ex);
    }

    return new HashSet(nameList);
  }//readCardNames()

  //public File getPath() {return path;}

  //return "C" for common, "U" for uncommon, "R" for rare, and
  //"not found" means that this card is not in common.txt, uncommon.txt, or rare.txt
  //no mythic rares buddy
  //
  //if a card name is in multiple files,
  //the lowest rarity where that card is found is returned
  //if a card name is found in common.txt and uncommon.txt "C" is returned
  public boolean isCommon(String cardName) {return commonList.contains(cardName);}
  public boolean isUncommon(String cardName) {return uncommonList.contains(cardName);}
  public boolean isRare(String cardName) {return rareList.contains(cardName);}

  //returns all the card names that are in cards.txt
  public ArrayList getAllCardNames()
  {
    return new ArrayList(cardTextMap.keySet());
  }//getAllCardNames()

  //reads from cards.txt
  //example:
  //ArrayList list = new ArrayList()
  //list.add("Shock")
  //list.add("R")
  //list.add("Instant");
  //list.add("Deals 2 damage to target creature or player")
  public ArrayList getCardInfo(String cardName)
  {
    if(! cardTextMap.containsKey(cardName))
    {
      Error.report("IO_Read : getCardInfo() error, card name not found - " +cardName);
      return new ArrayList();
    }

    return (ArrayList)cardTextMap.get(cardName);
  }//getCardInfo()

  public JPanel getCardPicture(String cardName)
  {
    String path = IO_Util.getCurrentDirectory() +"/res/pics/";

    String filename =  path + getPictureFilename(cardName);

    File check = new File(filename);
    if(check.exists())
      return new PicturePanel(filename);
//    else
//      System.out.println("IO_Read : getCardPicture() error, card picture filename not found - " +filename);

    return new JPanel();
  }//getCardPicture()

  //converts the String card name to the file
  //that holds the card's picture
  //removes spaces, dashes, etc...
  private String getPictureFilename(String cardName)
  {
    String in = cardName;

    StringBuffer out = new StringBuffer();
    char c;
    for(int i = 0; i < in.length(); i++)
    {
      c = in.charAt(i);
      if(c == ' ' || c == '-')
        out.append('_');
      else if(Character.isLetterOrDigit(c))
      {
        out.append(c);
      }
    }
    return out.toString().toLowerCase() +".jpg";
  }//getPictureFilename()

  public static void main(String[] args)
  {
    IO_Read io = new IO_Read("c:\\forge\\");
    System.out.println(io.getAllCardNames());
  }
}